/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 * Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextPane;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SpinnerNumberModel;

import tiimage.TIImage;
import tiimage.TIImageException;


public class CompressPanel extends JPanel {


	/**
	 * 
	 */
	private static final long serialVersionUID = 3671731119124224446L;

	private JProgressBar progressBar;

	PropertyChangeListener progressBarUpdater = new PropertyChangeListener() {

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if(evt.getNewValue() instanceof Double) {
				progressBar.setValue((int) (((Double) evt.getNewValue()).doubleValue()*100));
			}
		}

	};

	private JTextPane txtpnOutput;

	private JTextPane txtpnInput;

	private JSpinner spinWidth;

	private JCheckBox chckbxLineBreaks;
	
	/**
	 * Create the panel.
	 */
	public CompressPanel() {

		JScrollPane scrollPane = new JScrollPane();

		JScrollPane scrollPane_1 = new JScrollPane();

		JButton btnImprove = new JButton("Improve");
		btnImprove.setToolTipText("Compress the image");
		btnImprove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				new Thread(new Runnable() {
					
					@Override
					public void run() {
						TIImage loaded;
						try {
							loaded = new TIImage(txtpnInput.getText(), progressBarUpdater);
							String theImage = loaded.getString();
							if (theImage != null) {
								txtpnOutput.setText(String.format("\"%s\"", theImage));
								if (chckbxLineBreaks.isSelected()) {
									try {
										//Use a regex for finding complete bytes 
										String regex = String
												.format("(([^\\\\]|(\\\\(\\d|\"|\'){1,3})){1,%d}+)",
														spinWidth
																.getValue());
										String repl = "\"$1\" \\.\\.\n";

										theImage = theImage.replaceAll(
												regex, repl);
										theImage = theImage.substring(0,
												theImage.length() - 4);
										//theImage = theImage.replaceFirst("(.*)\\s*\\.\\.\\s*$", "$1");
									} catch (Exception e1) {
										e1.printStackTrace();
									}
									txtpnOutput.setText(theImage);
								} else {
									theImage = String.format("\"%s\"", theImage);
								}
								progressBar.setValue(100);
								Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(theImage), null);
								JOptionPane.showMessageDialog(CompressPanel.this, "String reduced by " + (txtpnInput.getText().length() - theImage.length()) + " characters and loaded to clipboard", "Loading complete", JOptionPane.INFORMATION_MESSAGE);
							}
						} catch (TIImageException e) {
							e.printStackTrace();
							JOptionPane.showMessageDialog(CompressPanel.this, "Could not load image: " + e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
						}
					}
				}).start();
			}
		});

		chckbxLineBreaks = new JCheckBox("Line Breaks");
		chckbxLineBreaks.setToolTipText("Distribute the string over several lines");

		spinWidth = new JSpinner();
		spinWidth.setToolTipText("Bytes per line");
		spinWidth.setModel(new SpinnerNumberModel(new Integer(100000), new Integer(100), null, new Integer(1)));

		progressBar = new JProgressBar();
		GroupLayout groupLayout = new GroupLayout(this);
		groupLayout.setHorizontalGroup(
				groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(Alignment.TRAILING, groupLayout.createSequentialGroup()
						.addContainerGap()
						.addGroup(groupLayout.createParallelGroup(Alignment.TRAILING)
								.addComponent(scrollPane_1, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 430, Short.MAX_VALUE)
								.addGroup(groupLayout.createSequentialGroup()
										.addComponent(scrollPane, GroupLayout.DEFAULT_SIZE, 336, Short.MAX_VALUE)
										.addPreferredGap(ComponentPlacement.RELATED)
										.addGroup(groupLayout.createParallelGroup(Alignment.TRAILING, false)
												.addComponent(chckbxLineBreaks, GroupLayout.DEFAULT_SIZE, 88, Short.MAX_VALUE)
												.addComponent(spinWidth)
												.addComponent(btnImprove, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
												.addComponent(progressBar, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 430, Short.MAX_VALUE))
												.addContainerGap())
				);
		groupLayout.setVerticalGroup(
				groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
						.addContainerGap()
						.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
								.addGroup(groupLayout.createSequentialGroup()
										.addComponent(btnImprove, GroupLayout.PREFERRED_SIZE, 71, GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(ComponentPlacement.RELATED)
										.addComponent(spinWidth, GroupLayout.PREFERRED_SIZE, 28, GroupLayout.PREFERRED_SIZE)
										.addPreferredGap(ComponentPlacement.UNRELATED)
										.addComponent(chckbxLineBreaks, GroupLayout.PREFERRED_SIZE, 18, GroupLayout.PREFERRED_SIZE))
										.addComponent(scrollPane, GroupLayout.PREFERRED_SIZE, 140, GroupLayout.PREFERRED_SIZE))
										.addGap(9)
										.addComponent(scrollPane_1, GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE)
										.addPreferredGap(ComponentPlacement.RELATED)
										.addComponent(progressBar, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
										.addGap(9))
				);

		txtpnOutput = new JTextPane();
		scrollPane_1.setViewportView(txtpnOutput);

		txtpnInput = new JTextPane();
		txtpnInput.setToolTipText("Paste your image here!");
		scrollPane.setViewportView(txtpnInput);
		setLayout(groupLayout);

		}
	}
