/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 * Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextPane;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import tiimage.TIImage;


public class ExportPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7429636814160432324L;

	private JTextPane txtpnImageOutput;

	private JFileChooser jf = new JFileChooser();

	private JLabel lblAlpha;

	private JSlider slider;

	private JSpinner spinWidth;

	private JScrollPane scrollPane;

	private JProgressBar progressBar;

	PropertyChangeListener progressBarUpdater = new PropertyChangeListener() {

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if(evt.getNewValue() instanceof Double) {
				progressBar.setValue((int) (((Double) evt.getNewValue()).doubleValue()*100));
			}
		}

	};

	private JCheckBox chckbxLineBreaks;

	/**
	 * Create the panel.
	 */
	public ExportPanel() {

		JButton btnLoadImage = new JButton("Load Image");
		btnLoadImage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int opt = jf.showOpenDialog(ExportPanel.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					new Thread(new Runnable() {

						@Override
						public void run() {
							BufferedImage img;
							try {
								img = ImageIO.read(jf.getSelectedFile());
								TIImage loaded = new TIImage(img, slider.getValue(), progressBarUpdater);
								String theImage = loaded.getString();
								if (theImage != null) {
									txtpnImageOutput.setText(String.format("\"%s\"", theImage));
									if (chckbxLineBreaks.isSelected()) {
										try {
											//Use a regex for finding complete bytes 
											String regex = String
													.format("(([^\\\\]|(\\\\(\\d|\"|\'){1,3})){1,%d}+)",
															spinWidth
																	.getValue());
											String repl = "\"$1\" \\.\\.\n";

											theImage = theImage.replaceAll(
													regex, repl);
											theImage = theImage.substring(0,
													theImage.length() - 4);
											//theImage = theImage.replaceFirst("(.*)\\s*\\.\\.\\s*$", "$1");
										} catch (Exception e1) {
											e1.printStackTrace();
										}
										txtpnImageOutput.setText(theImage);
									} else {
										theImage = String.format("\"%s\"", theImage);
									}
									progressBar.setValue(100);
									Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(theImage), null);
									JOptionPane.showMessageDialog(ExportPanel.this, "Image loaded successfully and loaded to clipboard", "Loading complete", JOptionPane.INFORMATION_MESSAGE);
								}
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}
					}).start();
				}
			}
		});

		scrollPane = new JScrollPane();


		lblAlpha = new JLabel("128");

		slider = new JSlider();
		slider.setToolTipText("Alpha value that determines whether a pixel will be drawn or not");
		slider.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent arg0) {
				lblAlpha.setText(String.format("%d", slider.getValue()));
			}
		});
		slider.setMaximum(255);
		slider.setValue(128);

		spinWidth = new JSpinner();
		spinWidth.setToolTipText("Number of bytes after which a new line will be started");
		spinWidth.setModel(new SpinnerNumberModel(new Integer(100000), new Integer(100), null, new Integer(1)));

		progressBar = new JProgressBar();
		
		chckbxLineBreaks = new JCheckBox("Line Breaks");
		chckbxLineBreaks.setToolTipText("Insert new line after specified number of bytes");

		GroupLayout groupLayout = new GroupLayout(this);
		groupLayout.setHorizontalGroup(
			groupLayout.createParallelGroup(Alignment.TRAILING)
				.addGroup(groupLayout.createSequentialGroup()
					.addContainerGap()
					.addGroup(groupLayout.createParallelGroup(Alignment.TRAILING)
						.addComponent(scrollPane, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
						.addComponent(btnLoadImage, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
						.addGroup(groupLayout.createSequentialGroup()
							.addComponent(slider, GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(lblAlpha)
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(spinWidth, GroupLayout.PREFERRED_SIZE, 80, GroupLayout.PREFERRED_SIZE)
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(chckbxLineBreaks))
						.addComponent(progressBar, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE))
					.addContainerGap())
		);
		groupLayout.setVerticalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addContainerGap()
					.addComponent(btnLoadImage)
					.addGap(12)
					.addGroup(groupLayout.createParallelGroup(Alignment.TRAILING)
						.addGroup(groupLayout.createParallelGroup(Alignment.BASELINE)
							.addComponent(chckbxLineBreaks)
							.addComponent(spinWidth, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
							.addComponent(lblAlpha, GroupLayout.PREFERRED_SIZE, 18, GroupLayout.PREFERRED_SIZE))
						.addComponent(slider, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.UNRELATED)
					.addComponent(scrollPane, GroupLayout.DEFAULT_SIZE, 183, Short.MAX_VALUE)
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(progressBar, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
					.addContainerGap())
		);

		txtpnImageOutput = new JTextPane();
		scrollPane.setViewportView(txtpnImageOutput);
		setLayout(groupLayout);

	}
}
