/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 * Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.filechooser.FileNameExtensionFilter;

import tiimage.TIImage;
import tiimage.TIImageException;
import javax.swing.JProgressBar;


public class ImportPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6955472348433460080L;

	private JTextField imageInput;

	BufferedImage image;

	private JScrollPane imgScrollView;

	private JButton btnSave;

	JFileChooser jf = new JFileChooser();
	private JPanel imgContainer;
	private JLabel label;

	PropertyChangeListener progressBarUpdater = new PropertyChangeListener() {

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			if(evt.getNewValue() instanceof Double) {
				progressBar.setValue((int) (((Double) evt.getNewValue()).doubleValue()*100));
			}
		}

	};

	private JProgressBar progressBar;

	/**
	 * Create the panel.
	 */
	public ImportPanel() {



		imageInput = new JTextField();
		imageInput.setToolTipText("Paste your image here");
		imageInput.setColumns(10);

		JButton btnImport = new JButton("Import");
		btnImport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					TIImage load = new TIImage(imageInput.getText(), progressBarUpdater);
					image = load.getImage();
					if(image  != null) {
						JLabel imgLabel = new JLabel();
						imgLabel.setIcon(new ImageIcon(image));



						imgContainer.removeAll();
						imgContainer = new JPanel();
						imgContainer.add(imgLabel);

						imgScrollView.setViewportView(imgContainer);

						btnSave.setEnabled(true);
					} else {
						imgScrollView.removeAll();
						btnSave.setEnabled(false);
					}
				} catch (TIImageException ex) {
					JOptionPane.showMessageDialog(ImportPanel.this, "Could not load image: " + ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					ex.printStackTrace();
					imgScrollView.removeAll();
					btnSave.setEnabled(false);
				}
			}

		});

		imgScrollView = new JScrollPane();

		btnSave = new JButton("Save");
		btnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jf.addChoosableFileFilter(new FileNameExtensionFilter("Portable Network Graphics", ".png"));
				//jf.addChoosableFileFilter(new FileNameExtensionFilter("JPEG", ".jpg", ".jpeg"));
				int opt = jf.showSaveDialog(ImportPanel.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					File f = jf.getSelectedFile();
					try {
						ImageIO.write(image, "png", f);
					} catch (IOException e1) {
						e1.printStackTrace();
						JOptionPane.showMessageDialog(ImportPanel.this, "Could not save Image" + e1.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
					}
				}

			}
		});
		btnSave.setEnabled(false);

		progressBar = new JProgressBar();
		
		GroupLayout groupLayout = new GroupLayout(this);
		groupLayout.setHorizontalGroup(
			groupLayout.createParallelGroup(Alignment.TRAILING)
				.addGroup(groupLayout.createSequentialGroup()
					.addContainerGap()
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING)
						.addComponent(progressBar, GroupLayout.DEFAULT_SIZE, 363, Short.MAX_VALUE)
						.addComponent(imgScrollView, GroupLayout.DEFAULT_SIZE, 363, Short.MAX_VALUE)
						.addComponent(imageInput, GroupLayout.DEFAULT_SIZE, 362, Short.MAX_VALUE))
					.addPreferredGap(ComponentPlacement.UNRELATED)
					.addGroup(groupLayout.createParallelGroup(Alignment.LEADING, false)
						.addComponent(btnImport, Alignment.TRAILING)
						.addComponent(btnSave, GroupLayout.PREFERRED_SIZE, 63, GroupLayout.PREFERRED_SIZE))
					.addContainerGap())
		);
		groupLayout.setVerticalGroup(
			groupLayout.createParallelGroup(Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
					.addContainerGap()
					.addGroup(groupLayout.createParallelGroup(Alignment.BASELINE)
						.addComponent(imageInput, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addComponent(btnImport))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(groupLayout.createParallelGroup(Alignment.TRAILING)
						.addComponent(btnSave)
						.addComponent(imgScrollView, GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(progressBar, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
					.addContainerGap())
		);

		imgContainer = new JPanel();
		imgScrollView.setViewportView(imgContainer);
		imgContainer.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

		label = new JLabel("");
		label.setIcon(new ImageIcon("C:\\Users\\Max\\Desktop\\affe.png"));
		//imgContainer.add(label);

		setLayout(groupLayout);

	}
}
